#include "data.h"
#include "peer.h"
#include "crypto.h"
#include "io.h"
#include "log.h"
#include "util.h"
#include "string.h"
#include "state.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

bool irc_user_store(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_IRC_USER);
  return str_store(path,irc_user);
}

bool irc_user_load(void){
  char path [PATH_BUF];
  char buf  [IRC_USER_LEN+WHITE_LEN+1+1]; //user\n\0

  FMT_PATH(path,"%s",FILE_IRC_USER);
  if(!file_exists(path)){
    log_err("required file '%s' doesn't exist. to create, run "
            "./" PROG_NAME " -u username",path);
    return 0;
  }

  if(!str_load(path,buf,sizeof(buf))){
    return 0;
  }

  if(!check_irc_user(buf)){
    return 0;
  }
  strzcpy(irc_user,buf,sizeof(irc_user));
  return 1;
}

bool irc_pass_store(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_IRC_PASS);
  return pass_store(path,irc_pass);
}

bool irc_pass_load(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_IRC_PASS);
  if(!file_exists(path)){
    log_err("required file '%s' doesn't exist. to create, run "
            "./" PROG_NAME " -p password",path);
    return 0;
  }
  return pass_load(path,irc_pass);
}

bool irc_port_store(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_IRC_PORT);
  return int_store(path,irc_port);
}

bool irc_port_load(void){
  char    path[PATH_BUF];
  int64_t i;

  FMT_PATH(path,"%s",FILE_IRC_PORT);
  if(!file_exists(path)){
    irc_port_store();
    return 1;
  }
  if(!int_load(path,&i)) return 0;
  if(!check_port(i,"IRC")) return 0;
  irc_port = i;
  return 1;
}

bool pest_port_store(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_PEST_PORT);
  return int_store(path,pest_port);
}

bool pest_port_load(void){
  char    path[PATH_BUF];
  int64_t i;

  FMT_PATH(path,"%s",FILE_PEST_PORT);
  if(!file_exists(path)){
    pest_port_store();
    return 1;
  }
  if(!int_load(path,&i)) return 0;
  if(!check_port(i,"pest")) return 0;
  pest_port = i;
  return 1;
}

bool scram_pass_store(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_SCRAM_PASS);
  return pass_store(path,scram_pass);
}

bool scram_pass_load(void){
  char path[PATH_BUF];
  FMT_PATH(path,"%s",FILE_SCRAM_PASS);
  if(!file_exists(path)){
    BUF_ZERO(scram_pass);
    return 1;
  }
  return pass_load(path,scram_pass);
}

