#include "pest.h"
#include "peer.h"
#include "packet.h"
#include "chain.h"
#include "crypto.h"
#include "log.h"
#include "time.h"
#include "util.h"
#include "config.h"
#include "state.h"

static void getdata_attempt(getdata_t *g,peer_t *chan){
  send_text(CMD_GETDATA,mono_sec(),g->hash,HASH_SIZE,chan,0);
  g->ntime = mono_nsec();
  g->attempt++;
}

static void del(getdata_t *g){
  ARR_DEL(gd,gd_cnt,g);
  gd_cnt--;
  memset_ffs(gd+gd_cnt,0x00,sizeof(*gd));
}

bool getdata_push(peer_t *p,peer_t *chan,hash_t const hash){
  getdata_t *g;

  assert(p || !chan);
  getdata_del(hash,0);

  if(gd_cnt >= GETDATA_MAX){
    irclog_err("getdata buffer full");
    return 0;
  }

  g           = &gd[gd_cnt++];
  g->peer_id  = p    ? p->id    : PEER_ID_INVALID;
  g->chan_id  = chan ? chan->id : PEER_ID_INVALID;
  g->attempt  = 0;
  hash_copy(g->hash,hash);

  getdata_attempt(g,chan);

  return 1;
}

bool getdata_push_cond(peer_t *p,peer_t *chan,hash_t const hash){
  if(hearsay_lookup(hash)) return 0;
  if(pending_lookup(hash)) return 0;
  if(getdata_lookup(hash)) return 0;
  return getdata_push(p,chan,hash);
}

static getdata_t *gd_lookup(hash_t const hash){
  for(size_t x = 0;x < gd_cnt;x++){
    getdata_t *g = gd+x;
    if(hash_equal(hash,g->hash)) return g;
  }
  return 0;
}

bool getdata_lookup(hash_t const hash){
  return gd_lookup(hash);
}

bool getdata_del(hash_t const hash,peer_t **peer){
  getdata_t *g;

  g = gd_lookup(hash);
  if(!g) return 0;
  if(peer){
    *peer = peer_lookup_id(g->peer_id);
  }
  del(g);

  return 1;
}

void getdata_del_peer(peer_t *p){
  for(size_t x = 0;x < gd_cnt;x++){
    getdata_t *g = gd+x;
    if(g->peer_id != p->id) continue;
    del(g);
    x--;
  }
}

void getdata_process(uint64_t ntime){
  for(size_t x = 0;x < gd_cnt;x++){
    getdata_t *g = gd+x;

    if(ntime < g->ntime+(gdtime*NSEC_MSEC)){
      continue;
    }

    if(g->attempt < (size_t)gdatt){
      peer_t *chan = peer_lookup_id(g->chan_id);
      getdata_attempt(g,chan);
    }else{
      del(g);
      x--;
    }
  }
}

//TODO: make les stupid
void getdata_retry(uint64_t time){
  if(time < gd_retry_last+gdint) return;
  gd_retry_last = time;

  for(size_t x = 0;x < peer_cnt;x++){
    peer_t *p = peer+x;
    //index 0 is considered "validest" of tails
    for(size_t y = 1;y < p->btail.cnt;y++){
      uint8_t const *h = p->btail.hash[y];
      if(hash_null(h)) continue;
      getdata_push_cond(p,0,h);
    }
    for(size_t y = 1;y < p->dtail.cnt;y++){
      uint8_t const *h = p->dtail.hash[y];
      if(hash_null(h)) continue;
      getdata_push_cond(p,p,h);
    }
  }
}

void getdata_handle(peer_t *p,hash_t hash){
  uint8_t   r[RED_SIZE];
  uint8_t  *m = r+MESSAGE_OFF;

  //TODO: only process if cpu avail

  if(mess_lookup(&mess,hash,m) == LOOKUP_FOUND){
    send_message(CMD_BROADCAST,r,hash,p,0);
  }else if(mess_lookup(&p->mess,hash,m) == LOOKUP_FOUND){
    send_message(CMD_DIRECT,r,hash,p,0);
  }

  BUF_ZERO(r);
}

