#include "pest.h"
#include "peer.h"
#include "packet.h"
#include "crypto.h"
#include "time.h"
#include "log.h"
#include "util.h"
#include "string.h"
#include "state.h"
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <assert.h>

void *hearsay_lookup(hash_t const hash){
  for(size_t x = 0;x < hs_cnt;x++){
    hearsay_t *h = hs+x;
    if(hash_equal(h->hash,hash)) return h;
  }
  return 0;
}

void hearsay_peer(void *_h,peer_t *p,uint8_t bounces){
  hearsay_t *h = _h;
  if(!hearsay_peered(h,p)){
    if(h->peer_cnt < PEER_MAX){
      h->peer[h->peer_cnt] = p->id;
    }
    h->peer_cnt++;
  }

  if(bounces < h->bounces){
    h->bounces = bounces;
  }
}

bool hearsay_peered(void *_h,peer_t *p){
  hearsay_t *h = _h;
  for(size_t x = 0;x < h->peer_cnt;x++){
    if(h->peer[x] == p->id) return 1;
  }
  return 0;
}

void *hearsay_push(hash_t const    hash,
                   uint8_t const  *red,
                   peer_t         *speaker,
                   peer_t         *p,
                   uint8_t         bounces){
  hearsay_t *h;

  if(hs_cnt >= HEARSAY_MAX){
    irclog_err("hearsay buffer full");
    return 0;
  }

  h = &hs[hs_cnt++];
  hash_copy(h->hash,hash);
  memcpy(h->red,red,sizeof(h->red));
  h->peer[0]  = p->id;
  h->peer_cnt = 1;
  h->speaker  = speaker ? speaker->id : PEER_ID_INVALID;
  h->ntime    = mono_nsec();
  h->bounces  = bounces;

  return h;
}

void hearsay_del(void *_h){
  hearsay_t *h = _h;
  ARR_DEL(hs,hs_cnt,h);
  hs_cnt--;
  memset_ffs(hs+hs_cnt,0x00,sizeof(*hs));
}

void hearsay_del_peer(peer_t *p){
  for(size_t x = 0;x < hs_cnt;x++){
    hearsay_t *h = hs+x;

    if(h->speaker == p->id){
      h->speaker = PEER_ID_INVALID;
    }

    for(size_t y = 0;y < h->peer_cnt;y++){
      peerid_t *q = h->peer+y;
      if(*q != p->id) continue;

      ARR_DEL(h->peer,h->peer_cnt,q);
      h->peer_cnt--;
      y--;

      if(h->peer_cnt == 0){
        hearsay_del(h);
        x--;
        break;
      }
    }
  }
}

static void fmt_hearsay(char       *buf,
                        size_t      buf_size,
                        char const *speaker,
                        hearsay_t  *h){

  if(h->peer_cnt <= HEARSAY_PARENS_MAX){
    handle_t parens[HEARSAY_PARENS_MAX];
    for(size_t x = 0;x < h->peer_cnt;x++){
      peer_t *p = peer_lookup_id(h->peer[x]);
      assert(p);
      strzcpy(parens[x],p->handle[0],sizeof(parens[x]));
    }
    fmt_handle_ext(buf,buf_size,speaker,parens,h->peer_cnt);
  }else{
    char num[HANDLE_BUF];
    snprintf(num,sizeof(num),"%zu",h->peer_cnt);
    fmt_handle_ext(buf,buf_size,speaker,&num,1);
  }
}

static void hearsay_message(hearsay_t *h){
  char        speaker[HANDLE_EXT_BUF];
  uint8_t    *m;
  char const *m_speaker;
  peer_t     *p;

  m         = h->red+MESSAGE_OFF;
  m_speaker = (char const *)(m+SPEAKER_OFF);

  fmt_hearsay(speaker,sizeof(speaker),m_speaker,h);
  if(p = peer_lookup_id(h->speaker)){
    //in-wot hearsay
    message_handle(m,h->hash,speaker,p,0,0,1,1);
  }else{
    //simple hearsay
    message_handle_simp(m,h->hash,speaker,0);
  }
  red_rebroadcast(h->red,h->hash,h->bounces);
}

void hearsay_process(uint64_t ntime){
  for(size_t x = 0;x < hs_cnt;x++){
    hearsay_t *h = hs+x;
    if(ntime < h->ntime+(hstime*NSEC_MSEC)){
      continue;
    }
    hearsay_message(h);
    hearsay_del(h);
    x--;
  }
}

