#ifndef IO_H
#define IO_H

#include "type.h"
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>
#include <stdio.h>
#include <sys/stat.h>

bool                file_exists       (char const *path);
bool                file_touch        (char const *path);
bool                file_delete       (char const *path);
bool                file_delete_zero  (char const *path);
bool                file_delete_nolog (char const *path);
bool                file_rename       (char const *from,char const *to);
bool                file_stat         (char const *path,struct stat *sb);

FILE               *file_open         (char const *path,char const *mode);
FILE               *file_open_zero    (char const *path,char const *mode);
bool                file_close        (FILE *f);
bool                file_seek         (FILE *f,size_t off,int origin);
bool                file_read         (FILE *f,void *data,size_t size);
bool                file_write        (FILE *f,void const *data,size_t size);
bool                file_gets         (FILE *f,void *buf,size_t buf_size);
bool                file_printf       (FILE *f,char const *fmt,...);

bool                mkdir_p           (char const *path);
bool                rm_rf             (char const *path);
bool                rm_rf_nolog       (char const *path);

bool                file_zero         (char const *path);
bool                file_zero_nolog   (char const *path);
bool                file_rand_nolog   (char const *path);
bool                dir_zero_nolog    (char const *path);
bool                dir_rand_nolog    (char const *path);

bool                str_store         (char const *path,char const *str);
bool                str_load          (char const *path,char *buf,size_t buf_size);
bool                int_store         (char const *path,int64_t  i);
bool                int_load          (char const *path,int64_t *i);
bool                bool_store        (char const *path,bool  b);
bool                bool_load         (char const *path,bool *b);
bool                hash_store        (char const *path,hash_t const h);
bool                hash_load         (char const *path,hash_t       h);
bool                hashlist_store    (char const *path,hash_t const *h,size_t  cnt);
bool                hashlist_load     (char const *path,hash_t       *h,size_t *cnt,size_t max);
bool                pass_store        (char const *path,uint8_t const *pass);
bool                pass_load         (char const *path,uint8_t       *pass);
bool                addr_store        (char const *path,addr_t const *addr);
bool                addr_load         (char const *path,addr_t       *addr);

void                disk_read         (char const *path);
void                disk_written      (char const *path);
void                disk_unsync       (void);

#endif

