#include "ircd.h"
#include "net.h"
#include "crypto.h"
#include "util.h"
#include "string.h"
#include "utf8.h"
#include "log.h"
#include "config.h"
#include "state.h"
#include <stdio.h>
#include <string.h>
#include <assert.h>

static void tcp_ev(unsigned e){
  switch(e){
  case TCP_CONNECT:
    if(ircd_func) ircd_func(IRCD_CONNECT,0);
    break;

  case TCP_DISCONNECT:
    if(ircd_func) ircd_func(IRCD_DISCONNECT,0);
    ircd_buf_p = ircd_buf;
    break;

  default:
    assert(0);
  }
}

void ircd_init(ircev_f func){
  ircd_func = func;
  tcp_init(tcp_ev);
}

void ircd_exit(void){
  tcp_cclose();
  tcp_lclose();
}

void ircd_ckill(void){
  tcp_cclose();
}

static bool parse_msg(ircmsg_t *m,char *c){
  char *t;
  char *u;

  memset(m,0x00,sizeof(*m));

  if(*c == ':'){
    c++;
    m->pre_name = c;

    u = strchr(c,' ');
    if(!u){
      log_err("IRC parse_msg: prefix space");
      return 0;
    }
    *u = 0;

    if(t = strchr(c,'!')){
      c           = t;
      *c++        = 0;
      m->pre_user = c;
    }

    if(t = strchr(c,'@')){
      c           = t;
      *c++        = 0;
      m->pre_host = c;
    }

    assert(u == strchr(c,0));
    c = strnotchr0(u+1,' ');
  }

  m->cmd  = c;
  c       = strchr(c,' ');
  if(!c){
    //RFC borken?
    goto end;
    //log_err("IRC parse_msg: first param space");
    //return 0;
  }
  *c++ = 0;

  while(1){
    c = strnotchr0(c,' ');
    if(*c == 0) break;

    if(m->argc >= IRCD_ARG_MAX){
      log_err("IRC parse_msg: max args exceeded");
      return 0;
    }

    m->argv[m->argc] = *c == ':' ? c+1 : c;
    m->argc++;
    if(*c == ':') break;

    c = strchr(c,' ');
    if(!c){
      //RFC borken?
      break;
      //log_err("IRC parse_msg: subsequent param space");
      //return 0;
    }
    *c++ = 0;
  }

end:
  str_tolower(m->cmd);
  //TODO: validate al shit
  return 1;
}

static void recv_msg(char *cmd){
  ircmsg_t m;
  if(!parse_msg(&m,cmd)){
    log_err("failed to parse IRC message");
    return;
  }
  if(ircd_func) ircd_func(IRCD_MESSAGE,&m);
}

static bool ircd_recv(void){
  char   *buf   = ircd_buf;
  char   *buf_e = ircd_buf+sizeof(ircd_buf);
  char   *buf_p = ircd_buf_p;
  char   *r;
  size_t  n;

  n = tcp_recv(&ircd_src_ip,
               &ircd_src_port,
               irc_port,
               (uint8_t *)buf_p,
               buf_e-buf_p);
  if(!n) return 0;

  if(memchr(buf_p,0x00,n)){
    log_err("IRC command contains null byte");
    ircd_ckill();
    return 0;
  }

  buf_p += n;

  while(r = str_nstr(buf,"\r\n",buf_p-buf)){
    *r  = 0;
    r  += 2;

    if(utf8_valid(buf)){
      recv_msg(buf);
    }else{
      log_err("IRC command contains invalid UTF-8 sequence");
    }

    assert(r <= buf_p);
    memmove(buf,r,buf_p-r);
    buf_p = buf+(buf_p-r);
  }

  if(buf_p-buf >= IRC_MSG_LEN){
    log_err("IRC command too long");
    ircd_ckill();
    return 0;
  }

  ircd_buf_p = buf_p;
  return 1;
}

void ircd_tick(void){
  for(size_t x = 0;x < IRCD_RECV_MAX;x++){
    if(!ircd_recv()) break;
  }
}

size_t ircd_send(char const *str,size_t len){
  return tcp_send((uint8_t const *)str,len);
}

size_t ircd_sendf(char const *fmt,...){
  va_list args;
  size_t  n;

  va_start(args,fmt);
  n = ircd_vsendf(fmt,args);
  va_end(args);
  return n;
}

size_t ircd_vsendf(char const *fmt,va_list args){
  char    buf[IRC_MSG_BUF];
  char   *b;
  int     n;
  size_t  r;

  n = vsnprintf(buf,sizeof(buf),fmt,args);
  if(n < 0){
    BUF_ZERO(buf);
    log_errno("vsnprintf");
    return 0;
  }

  b = buf+n;
  b = strpcpy(b,"\r\n",buf+sizeof(buf));
  r = ircd_send(buf,b-buf);
  memset_ffs(buf,0x00,b-buf);
  return r;
}

uint32_t ircd_getip(void){
  return ircd_src_ip;
}

