#include "crypto.h"
#include "spec.h"
#include "string.h"
#include "def.h"
#include "log.h"
#include "state.h"
#include <assert.h>

bool key_gen(uint8_t *key){
  if(!rng_read(rng_slow_fd,key,KEY_SIZE)){
    irclog_err("could not generate key, rng read failed");
    return 0;
  }

  if(entropy_missing(key,KEY_SIZE)){
    irclog_err("could not generate key, entropy missing");
    return 0;
  }

  return 1;
}

void b64_from_key(char *buf,size_t buf_size,uint8_t const *key){
  size_t n = base64_enc(key,KEY_SIZE,buf,buf_size);
  assert(n == BASE64_ENC_SIZE(KEY_SIZE,1)); (void)n;
}

bool key_from_b64(uint8_t *key,char const *b64_key){
  size_t n;

  if(!str_valid(b64_key,B64_KEY_LEN,B64_KEY_LEN,BASE64_CHARSET)){
    return 0;
  }

  n = base64_dec(b64_key,key,KEY_SIZE,0);
  assert(n == KEY_SIZE); (void)n;
  return 1;
}

void key_zero(uint8_t *key){
  memset_ffs(key,0x00,KEY_SIZE);
}

void key_copy(uint8_t *dst,uint8_t const *src){
  memcpy(dst,src,KEY_SIZE);
}

void key_xor(uint8_t *dst,uint8_t const *a,uint8_t const *b){
  for(size_t x = 0;x < KEY_SIZE;x++){
    dst[x] = a[x]^b[x];
  }
}

bool key_equal(uint8_t const *a,uint8_t const *b){
  return memcmp(a,b,KEY_SIZE) == 0;
}

