#include "data.h"
#include "state.h"
#include "io.h"
#include "log.h"
#include "util.h"
#include "string.h"
#include <stdio.h>
#include <string.h>

static bool killfile_add_internal(char const *handle,bool boot){
  log_f   out = boot ? log_void : irclog_out;
  log_f   err = boot ? log_err  : irc_err;

  if(killfile_cnt >= KILLFILE_MAX){
    err("max kills reached");
    return 0;
  }

  if(!handle_valid(handle)){
    err("invalid handle: '%s'",handle);
    return 0;
  }

  if(killfile_lookup(handle)){
    err("handle '%s' already exists in killfile",handle);
    return 0;
  }

  strzcpy(killfile[killfile_cnt],handle,sizeof(killfile[killfile_cnt]));
  killfile_cnt++;

  if(!boot) killfile_store();

  out("handle '%s' added to killfile",handle);
  return 1;
}

bool killfile_add(char const *handle){
  return killfile_add_internal(handle,0);
}

bool killfile_del(char const *handle){
  handle_t *h;

  if(!handle_valid(handle)){
    irc_err("invalid handle: '%s'",handle);
    return 0;
  }

  h = killfile_lookup(handle);
  if(!h){
    irc_err("handle '%s' not found in killfile",handle);
    return 0;
  }

  ARR_DEL(killfile,killfile_cnt,h);
  killfile_cnt--;
  killfile_store();
  irclog_out("handle '%s' deleted from killfile",handle);
  return 1;
}

handle_t *killfile_lookup(char const *handle){
  for(size_t x = 0;x < killfile_cnt;x++){
    handle_t *h = killfile+x;
    if(strncmp(*h,handle,HANDLE_LEN) == 0) return h;
  }
  return 0;
}

void killfile_print(void){
  if(killfile_cnt == 0){
    irc_out("killfile is empty");
    return;
  }
  irc_out("--- BEGIN KILLFILE ---");
  for(size_t x = 0;x < killfile_cnt;x++){
    irc_out("%s",killfile+x);
  }
  irc_out("--- END KILLFILE ---");
}

bool killfile_load(void){
  char    path[PATH_BUF];
  FILE   *f;
  char    buf[HANDLE_BUF+1+1]; //\n+1 for detecting invalid handle
  char   *b;

  FMT_PATH(path,"%s",FILE_KILLFILE);
  if(!file_exists(path)) return 1;
  f = file_open(path,"rb");
  if(!f) return 0;

  while(file_gets(f,buf,sizeof(buf))){
    b = str_trim(buf);
    if(!*b) continue;

    if(!killfile_add_internal(b,1)){
      file_close(f);
      return 0;
    }
  }

  if(ferror(f)){
    file_close(f);
    return 0;
  }

  file_close(f);
  disk_read(path);
  return 1;
}

bool killfile_store(void){
  char  path[PATH_BUF];
  FILE *f;

  FMT_PATH(path,"%s",FILE_KILLFILE);
  if(killfile_cnt == 0){
    if(!file_delete_zero(path)){
      disk_unsync();
      return 0;
    }
    return 1;
  }

  f = file_open_zero(path,"wb");
  if(!f){
    disk_unsync();
    return 0;
  }

  for(size_t x = 0;x < killfile_cnt;x++){
    if(!file_printf(f,"%s\n",killfile[x])){
      file_close(f);
      disk_unsync();
      return 0;
    }
  }

  if(!file_close(f)){
    disk_unsync();
    return 0;
  }
  disk_written(path);
  return 1;
}

