static char        *p_banner = banner;
static char        *p_acaddr = acaddr;
static char        *p_master = master;

static knob_t const knob[] = {

  { .data       = &setport,
    .name       = FILE_SETPORT,
    .desc       = "update source port in addition to ip address in AT entries",
    .type       = KNOB_BOOL,
    .def.b      = 1,
  },

  { .data       = &cutoff,
    .name       = FILE_CUTOFF,
    .desc       = "bounce cutoff for rejecting packets",
    .type       = KNOB_INT,
    .def.i      = 5,
    .min        = CUTOFF_MIN,
    .max        = CUTOFF_MAX,
  },

  { .data       = &hstime,
    .name       = FILE_HSTIME,
    .desc       = "hearsay buffer timeout in milliseconds",
    .type       = KNOB_INT,
    .def.i      = 1000,
    .min        = 0,
    .max        = KNOB_MAX,
  },

  { .data       = &gdtime,
    .name       = FILE_GDTIME,
    .desc       = "getdata attempt timeout in milliseconds "
                  "(actual = gdtime*gdatt)",
    .type       = KNOB_INT,
    .def.i      = 500,
    .min        = 0,
    .max        = KNOB_MAX,
  },

  { .data       = &gdatt,
    .name       = FILE_GDATT,
    .desc       = "number of getdata attempts before give up",
    .type       = KNOB_INT,
    .def.i      = 2,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &gdint,
    .name       = FILE_GDINT,
    .desc       = "interval in seconds between attempts "
                  "to repair broken chains",
    .type       = KNOB_INT,
    .def.i      = 10,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &rkrecv,
    .name       = FILE_RKRECV,
    .desc       = "processing of incoming rekey requests",
    .type       = KNOB_BOOL,
    .def.b      = 0,
  },

  { .data       = &rksend,
    .name       = FILE_RKSEND,
    .desc       = "automatic rekeying requests",
    .type       = KNOB_BOOL,
    .def.b      = 0,
  },

  { .data       = &rkint,
    .name       = FILE_RKINT,
    .desc       = "automatic rekeying request interval in hours",
    .type       = KNOB_INT,
    .def.i      = 24,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &rktime,
    .name       = FILE_RKTIME,
    .desc       = "rekeying ritual timeout in seconds",
    .type       = KNOB_INT,
    .def.i      = 10,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &igint,
    .name       = FILE_IGINT,
    .desc       = "average interval in milliseconds between "
                  "ignore messages sent",
    .type       = KNOB_INT,
    .def.i      = 1000,
    .min        = 100,
    .max        = 10000,
  },

  { .data       = &p_banner,
    .name       = FILE_BANNER,
    .desc       = "human-readable description of station "
                  " ('' = prog name and version)",
    .type       = KNOB_STR,
    .def.s      = "",
    .min        = 0,
    .max        = sizeof(banner)-1,
  },

  { .data       = &baint,
    .name       = FILE_BAINT,
    .desc       = "banner broadcast interval in seconds",
    .type       = KNOB_INT,
    .def.i      = 600,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &p_acaddr,
    .name       = FILE_ACADDR,
    .desc       = "if not '', ip:port that will be "
                  "transmitted in periodic address casts",
    .type       = KNOB_STR,
    .def.s      = "",
    .min        = 0,
    .max        = sizeof(acaddr)-1,
  },

  { .data       = &acint,
    .name       = FILE_ACINT,
    .desc       = "address cast interval in seconds",
    .type       = KNOB_INT,
    .def.i      = 600,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &irctime,
    .name       = FILE_IRCTIME,
    .desc       = "timeout in seconds before peer not "
                  "sending packets parts channel",
    .type       = KNOB_INT,
    .def.i      = 60,
    .min        = 1,
    .max        = KNOB_MAX,
  },

  { .data       = &p_master,
    .name       = FILE_MASTER,
    .desc       = "if not '', enables slave mode and this "
                  "handle identifies master peer",
    .type       = KNOB_STR,
    .def.s      = "",
    .min        = 0,
    .max        = sizeof(master)-1,
  },

};

