#include "log.h"
#include "string.h"
#include "time.h"
#include "io.h"
#include "util.h"
#include "state.h"
#include "config.h"
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#define LOG_STAMP_BUF       256

static void timestamp(FILE *f,uint64_t time){
  char ts[LOG_STAMP_BUF];
  time_str(ts,sizeof(ts),LOG_STAMP_FMT" ",time);
  fputs(ts,f);
}

static void _log_vout(FILE *f,char const *fmt,va_list args,uint64_t time){
  timestamp(f,time);
  vfprintf(f,fmt,args);
  fputc('\n',f);
  fflush(f);
}

static void _log_verr(FILE *f,char const *fmt,va_list args,uint64_t time){
  timestamp(f,time);
  fputs("error: ",f);
  vfprintf(f,fmt,args);
  fputc('\n',f);
  fflush(f);
}

bool log_open(char const *path){
  log_fp = file_open(path,"ab");
  return log_fp != 0;
}

bool log_close(void){
  if(!log_fp) return 1;
  bool r = file_close(log_fp);
  log_fp = 0;
  return r;
}

void log_void(char const *fmt,...){
  (void)fmt;
}

void log_out(char const *fmt,...){
  va_list args;
  va_start(args,fmt);
  log_vout(fmt,args);
  va_end(args);
}

void log_vout(char const *fmt,va_list args){
  uint64_t time = time_sec();
  if(log_fp){
    va_list args2;
    va_copy(args2,args);
    _log_vout(log_fp,fmt,args2,time);
    va_end(args2);
  }
  _log_vout(stderr,fmt,args,time);
}

void log_err(char const *fmt,...){
  va_list args;
  va_start(args,fmt);
  log_verr(fmt,args);
  va_end(args);
}

void log_verr(char const *fmt,va_list args){
  uint64_t time = time_sec();
  if(log_fp){
    va_list args2;
    va_copy(args2,args);
    _log_verr(log_fp,fmt,args2,time);
    va_end(args2);
  }
  _log_verr(stderr,fmt,args,time);
}

void log_errno(char const *call){
  log_err("%s(): %s",call,strerror(errno));
}

