#include "chain.h"
#include "crypto.h"
#include "io.h"
#include "log.h"
#include "util.h"
#include "config.h"
#include "string.h"
#include <string.h>

void mess_init(mess_t *c){
  *c->path    = 0;
  c->idx_cnt  = 0;
  c->msg_cnt  = 0;
  c->idx_ecnt = 0;
  c->msg_ecnt = 0;
}

void mess_path(mess_t *c,char const *path){
  strzcpy(c->path,path,sizeof(c->path));
}

bool mess_load(mess_t *c){
  if(!mkdir_p(c->path)) return 0;

  if(!idx_cnt (c,&c->idx_cnt )) return 0;
  if(!idx_ecnt(c,&c->idx_ecnt)) return 0;
  if(!msg_cnt (c,&c->msg_cnt )) return 0;
  if(!msg_ecnt(c,&c->msg_ecnt)) return 0;

  //TODO: chek if idx and msg total count matches
  //TODO: log al operations

#ifdef LOG_MESS
  log_out("mess loaded: '%s' (message count: %zu)",
          c->path,msg_count(c));
#endif

  return 1;
}

bool mess_push(mess_t        *c,
               hash_t const   hash,
               uint8_t const *m){

  uint64_t index;

  assert(mess_lookup(c,hash,0) != LOOKUP_FOUND);

#ifndef NDEBUG
  hash_t mhash;
  hash_data(mhash,m,MESSAGE_SIZE);
  assert(hash_equal(mhash,hash));
#endif

  if(!msg_ins(c,m,&index))    return 0;
  if(!idx_ins(c,hash,index))  return 0;

#ifdef LOG_MESS
  char hash_hex[HASH_HEX_BUF];
  hash_str_log(hash_hex,sizeof(hash_hex),hash);
  log_out("%s pushed to mess: '%s'",hash_hex,c->path);
#endif

  return 1;
}

unsigned mess_lookup(mess_t        *c,
                     hash_t const   hash,
                     uint8_t       *m){

  uint64_t index;

  if(hash_null(hash)) return LOOKUP_NOTFOUND;

  switch(idx_lookup(c,hash,&index)){
  case LOOKUP_FOUND:
    break;

  case LOOKUP_NOTFOUND:
#ifdef LOG_MESS
    ;char hash_hex[HASH_HEX_BUF];
    hash_str_log(hash_hex,sizeof(hash_hex),hash);
    log_out("%s not found in mess: '%s'",hash_hex,c->path);
#endif
    return LOOKUP_NOTFOUND;

  case LOOKUP_IO:
    return LOOKUP_IO;
  }

  if(!msg_get(c,m,index)){
    if(m) memset_ffs(m,0x00,MESSAGE_SIZE);
    return LOOKUP_IO;
  }

#ifdef LOG_MESS
  char hash_hex[HASH_HEX_BUF];
  hash_str_log(hash_hex,sizeof(hash_hex),hash);
  log_out("%s found in mess: '%s'",hash_hex,c->path);
#endif

  return 1;
}

