#include "pest.h"
#include "packet.h"
#include "peer.h"
#include "log.h"
#include "time.h"
#include "crypto.h"
#include "serial.h"
#include "util.h"
#include "string.h"
#include "config.h"
#include "state.h"

#define PENDING_TIMEOUT (gdtime*gdatt+10)

static void del(pending_t *e){
  ARR_DEL(pen,pen_cnt,e);
  pen_cnt--;
  memset_ffs(pen+pen_cnt,0x00,sizeof(*pen));
}

static void send_chain(pending_t *e,bool timeout,bool top){
  hash_t            hash;
  peer_t           *p;
  peer_t           *chan;
  static uint64_t   stamp;

  hash_copy(hash,e->hash);

  p     = peer_lookup_id(e->peer_id);
  chan  = peer_lookup_id(e->chan_id);
  assert(p);
  if(top) getdata_del(e->hash,0);
  message_handle(e->m,e->hash,e->speaker,p,chan,1,0,1);
  read_u64(e->m+TIMESTAMP_OFF,&stamp);
  del(e);

again:
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t      *f       = pen+x;
    uint8_t const  *f_prev  = f->m+SELFCHAIN_OFF;

    if(!f->tail) continue;
    if(!hash_equal(f_prev,hash)) continue;

    send_chain(f,timeout,0);
    goto again;
  }

  if(top){
    //set last stamp to last message in chain
    console_stamp(chan,stamp);  
  }
}

//TODO: undumbtarded algo for keping trak of bigness
static size_t bigness(pending_t *e){
  size_t n = 1;
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t      *f       = pen+x;
    uint8_t const  *f_prev  = f->m+SELFCHAIN_OFF;

    if(e == f) continue;
    if(!f->tail) continue;
    if(!hash_equal(f_prev,e->hash)) continue;

    n += bigness(f);
  }
  return n;
}

static void force_timeout(void){
  size_t      big_max = 0;
  pending_t  *big     = 0; 

  for(size_t x = 0;x < pen_cnt;x++){
    pending_t  *e = pen+x;
    size_t      e_big;

    if(e->tail) continue;

    e_big = bigness(e);
    if(e_big > big_max){
      big     = e;
      big_max = e_big;
    }
  }

  assert(big);
  send_chain(big,1,1);
}

bool pending_dontfull(void){
  if(pen_cnt >= PENDING_MAX){
    irclog_out("pending message buffer full, forcing timeout");
    force_timeout();
    return 1;
  }
  return 0;
}

bool pending_lookup(hash_t const hash){
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t *e = pen+x;
    if(hash_equal(e->hash,hash)) return 1;
  }
  return 0;
}

static bool test_heads(hash_t const prev){
  return pending_lookup(prev);
}

static void test_tails(hash_t const hash){
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t      *e       = pen+x;
    uint8_t const  *e_prev  = e->m+SELFCHAIN_OFF;
    if(hash_equal(e_prev,hash)){
      e->tail = 1;
    }
  }
}

bool pending_push(uint8_t const  *m,
                  hash_t const    hash,
                  char const     *speaker,
                  peer_t         *p,
                  peer_t         *chan){

  pending_t      *e;
  uint8_t const  *prev = m+SELFCHAIN_OFF;

  assert(pen_cnt < PENDING_MAX);

  e           = &pen[pen_cnt++];
  e->peer_id  = p->id;
  e->chan_id  = chan ? chan->id : PEER_ID_INVALID;
  e->ntime    = mono_nsec();
  hash_copy(e->hash,hash);
  memcpy(e->m,m,sizeof(e->m));
  strzcpy(e->speaker,speaker,sizeof(e->speaker));

  //see if this is continuation in > dir
  e->tail = test_heads(prev);

  //see if this is continuation in < dir
  test_tails(hash);

  if(!e->tail && !hearsay_lookup(e->m+SELFCHAIN_OFF)){
    getdata_push(p,chan,e->m+SELFCHAIN_OFF);
  }

  return 1;
}

//TODO: this shit is entirely stupid
bool pending_send_chain(hash_t const prev){
  bool r = 0;
again:
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t *e = pen+x;
    if(hash_equal(e->m+SELFCHAIN_OFF,prev)){
      send_chain(e,0,1);
      r = 1;
      goto again;
    }
  }
  return r;
}

void pending_del_peer(peer_t *p){
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t *e = pen+x;
    if(e->chan_id == p->id){
      del(e);
      x--;
    }
  }
}

void pending_process(uint64_t ntime){
again:
  for(size_t x = 0;x < pen_cnt;x++){
    pending_t *e = pen+x;

    if(ntime < e->ntime+(PENDING_TIMEOUT*NSEC_MSEC)){
      continue;
    }

    if(e->tail){
      continue;
    }

    send_chain(e,1,1);
    goto again;
  }
}

