#ifndef PEST_H
#define PEST_H

#include "type.h"
#include <stdint.h>
#include <stdbool.h>

bool                pest_init           (void);
void                pest_tick           (uint64_t time,uint64_t ntime,bool active);
bool                pest_broadcast      (char const *msg);
bool                pest_direct         (char const *handle,char const *msg);

void                knob_display        (char const *name,bool alt);
void                knob_set            (char const *name,char const *val);
bool                knob_load           (void);

bool                dedup_load          (void);
bool                dedup_lookup        (hash_t const hash);
bool                dedup_push          (hash_t const hash,uint64_t time);
void                dedup_clean         (uint64_t time);

bool                getdata_push        (peer_t *p,peer_t *chan,hash_t const hash);
bool                getdata_push_cond   (peer_t *p,peer_t *chan,hash_t const hash);
bool                getdata_lookup      (hash_t const hash);
bool                getdata_del         (hash_t const hash,peer_t **peer);
void                getdata_del_peer    (peer_t *p);
void                getdata_process     (uint64_t ntime);
void                getdata_retry       (uint64_t time);
void                getdata_handle      (peer_t *p,hash_t hash);

bool                pending_dontfull    (void);
bool                pending_lookup      (hash_t const hash);
bool                pending_push        (uint8_t const *m,hash_t const hash,char const *speaker,peer_t *p,peer_t *chan);
bool                pending_send_chain  (hash_t const prev);
void                pending_del_peer    (peer_t *p);
void                pending_process     (uint64_t ntime);

void               *hearsay_lookup      (hash_t const hash);
void                hearsay_peer        (void *h,peer_t *p,uint8_t bounces);
bool                hearsay_peered      (void *h,peer_t *p);
void               *hearsay_push        (hash_t const hash,uint8_t const *red,peer_t *speaker,peer_t *p,uint8_t bounces);
void                hearsay_del         (void *h);
void                hearsay_del_peer    (peer_t *p);

void                hearsay_process     (uint64_t ntime);

void                banner_handle       (peer_t *p,uint8_t const *m);
void                banner_broadcast    (uint64_t time);

void                rekey_init          (rekey_t *rk);
bool                rekey_active        (rekey_t *rk);
bool                rekey_keyexists     (rekey_t *rk);
void                rekey_initiate      (peer_t *p,bool log);
bool                rekey_handle_offer  (peer_t *p,uint8_t const *offer);
bool                rekey_handle_slice  (peer_t *p,uint8_t const *slice);
bool                rekey_handle_ignore (peer_t *p,pkey_t *pk,uint64_t time);
bool                rekey_handle_packet (peer_t *p,pkey_t *pk);
void                rekey_broadcast     (uint64_t time);
void                rekey_clean         (uint64_t time);

void                addrcast_broadcast  (uint64_t time);
void                addrcast_handle     (uint8_t *r,hash_t const hash,uint8_t bounces,char const *speaker);

bool                ignore_send         (peer_t *chan,pkey_t *pk);
void                ignore_broadcast    (uint64_t time);

void                message_handle      (uint8_t const *m,hash_t const hash,char const *speaker,peer_t *p,peer_t *chan,bool stampold,bool pend,bool display);
void                message_handle_simp (uint8_t const *m,hash_t const hash,char const *speaker,bool stampold);
bool                broadcast_handle    (peer_t *p,peer_t *gd_peer,bool immediate,bool pmaster,uint8_t *r,hash_t const hash,uint8_t bounces,char const *speaker);
bool                direct_handle       (peer_t *p,peer_t *gd_peer,bool immediate,uint8_t *r,hash_t const hash,uint8_t bounces,char const *speaker);

void                console_send        (peer_t *p,peer_t *chan,char const *speaker,uint8_t const *m,bool stampold);
void                console_stamp       (peer_t *p,uint64_t stamp);

#endif

