#include "packet.h"
#include "pest.h"
#include "chain.h"
#include "addr.h"
#include "peer.h"
#include "net.h"
#include "crypto.h"
#include "time.h"
#include "log.h"
#include "io.h"
#include "serial.h"
#include "util.h"
#include "string.h"
#include "config.h"
#include "state.h"
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

static bool stamp_stale(uint64_t time,uint64_t stamp){
  return stamp < time-TIMESTAMP_MAXDIFF ||
         stamp > time+TIMESTAMP_MAXDIFF;
}

static bool find_key(uint8_t const *b,peer_t **p_out,pkey_t **pk_out){
  uint16_t peer_idx[peer_cnt];

  if(!index_shuffle(peer_idx,peer_cnt)){
    irclog_err("could not shuffle index, "
               "black packet cannot be processed");
    return 0;
  }

  for(size_t x = 0;x < peer_cnt;x++){
    peer_t *p = peer+peer_idx[x];
    if(p->paused) continue;

    for(size_t y = 0;y < p->key_cnt;y++){
      pkey_t *pk = p->key+y;
      if(!black_verify(b,pk->key.sub.sign)) continue;
      *p_out  = p;
      *pk_out = pk;
      return 1;
    }
  }
  return 0;
}

static void at_update(peer_t *p,addr_t *addr){
  if(setport){
    if(!addr_equal(&p->addr,addr)){
      peer_addr_set(p,addr,0);
    }
  }else if(p->addr.ip != addr->ip){
    addr_t p_addr;
    p_addr.ip   = addr->ip;
    p_addr.port = p->addr.port;
    peer_addr_set(p,&p_addr,0);
  }
}

//TODO: this func needs to become smaller or smth
bool pest_recv(uint64_t time){
  uint8_t       b           [BLACK_SIZE+1]; //+1 for detecting oversize
  char          speaker     [HANDLE_BUF];
  hash_t        hash;
  uint8_t      *r;
  uint8_t      *m;
  uint8_t      *t;
  size_t        n;
  addr_t        addr;
  uint8_t       bounces;
  uint8_t       version;
  uint8_t       command;
  uint64_t      stamp;
  peer_t       *p;
  pkey_t       *pk;
  char         *handle;
  void         *h;
  bool          immediate;
  bool          pmaster;
  bool          gd;
  peer_t       *gd_peer;

  //TMP xtrem spam test
  //static int ass = 0;
  ////if(irc_ready && ++ass > 1){
  ////if(irc_ready && ++ass > 14){
  //if(irc_ready && ++ass > 20){
  ////if(irc_ready && ++ass > 40){
    //char cunt[45];
    //rng_read(rng_fast_fd,cunt,sizeof(cunt));
    //for(size_t x = 0;x < sizeof(cunt);x++){
      //if(cunt[x] < 0x20 || cunt[x] > 0x7E) cunt[x] = '.';
    //}
    //cunt[sizeof(cunt)-1] = 0;
    //pest_broadcast(cunt);
    //ass = 0;
  //}

  n = udp_recv(&addr.ip,&addr.port,pest_port,b,sizeof(b));
  if(!n) return 0;

  //TMP xtrem shity conection test
  //if(rand()%5 == 0) return 0;

  if(n != BLACK_SIZE){
    log_martian(&addr,1);
    return 1;
  }

  if(!find_key(b,&p,&pk)){
    log_martian(&addr,0);
    return 1;
  }

  r = b+RED_OFF;
  m = r+MESSAGE_OFF;
  t = m+TEXT_OFF;

  black_deciph(r,b,pk->ciph_serp);
  hash_data(hash,m,MESSAGE_SIZE);
  peer_pak_stamp(p,time);

  read_u8   (r+BOUNCES_OFF,   &bounces);
  read_u8   (r+VERSION_OFF,   &version);
  read_u8   (r+COMMAND_OFF,   &command);
  read_u64  (m+TIMESTAMP_OFF, &stamp);

  handle = p->handle[0];

  //TODO: vat do with this
  //if(version > PEST_VERSION){
    //log_out("protocol version 0x%02X (current: 0x%02X) "
            //"packet rejected from peer '%s'",
            //version,PEST_VERSION,handle);
    //goto clear;
  //}

  if(command >= CMD_UNDEF_MIN &&
     command <= CMD_UNDEF_MAX){
    irclog_err("unknown pest command (0x%02X) "
               "received from peer '%s'",
               command,handle);
    goto clear;
  }

  if(bounces > cutoff){
    log_cutoff(hash,command,p,&addr,bounces,time,stamp);
    goto clear;
  }

  //TODO: vtf do with netchain
  gd = getdata_del(hash,&gd_peer);
  if(gd){
    //TMP, vat do with !gd_peer response?
    assert(gd_peer);
  }

  //skip stamp stale check if expecting getdata
  if(!gd && stamp_stale(time,stamp)){
    log_stale(hash,command,p,&addr,time,stamp);
    goto clear;
  }

  if(!gd && command == CMD_BROADCAST){
    h = hearsay_lookup(hash);
  }else{
    h = 0;
  }

  //dont dedup or push (cuz alredy in) if in hearsay buffer
  if(!h){
    //dont dedup if getdata response, in some extreme cases
    //have to throw away packets and no good way to delete
    //data from middle of dedup buffer. instead, make sure
    //to never send getdata for message that exists somewhere
    if(!gd && dedup_lookup(hash)){
      log_dupe(hash,command,p,&addr,time,stamp);
      goto clear;
    }

    if(!dedup_push(hash,time)){
      log_dedup_full(hash,command,p,&addr,time,stamp);
      goto clear;
    }
  }

  peer_stamp(p,time);
  peer_key_stamp(p,pk,time);
  at_update(p,&addr);

  log_packet(hash,command,p,&addr,0,time,stamp);

  cpy_speaker(speaker,sizeof(speaker),m);
  immediate = peer_handle_lookup(p,speaker);
  pmaster   = *master && peer_handle_lookup(p,master);

  if(h){
    if(immediate || pmaster){
      //replace hearsay with immediate version
      hearsay_del(h);
    }else{
      //add peer to message already in hearsay buffer
      hearsay_peer(h,p,bounces);
      goto clear;
    }
  }

  //TODO: should this be handled only for BROADCAST/DIRECT? add to chain?
  if(killfile_lookup(speaker)){
    log_out("speaker '%s' in killfile, ignoring packet",speaker);
    goto clear;
  }

  switch(command){
  case CMD_BROADCAST:
    broadcast_handle(p,gd ? gd_peer : 0,immediate,pmaster,
                     r,hash,bounces,speaker);
    break;

  case CMD_DIRECT:
    direct_handle(p,gd ? gd_peer : 0,immediate,
                  r,hash,bounces,speaker);
    break;

  case CMD_BANNER:
    banner_handle(p,m);
    break;

  case CMD_GETDATA:
    getdata_handle(p,t);
    break;

  case CMD_KEYOFFER:
    rekey_handle_offer(p,t);
    break;

  case CMD_KEYSLICE:
    rekey_handle_slice(p,t);
    break;

  case CMD_ADDRESSCAST:
    addrcast_handle(r,hash,bounces,speaker);
    break;

  case CMD_IGNORE:
    rekey_handle_ignore(p,pk,time);
    break;

  default:
    assert(0);
  }

  rekey_handle_packet(p,pk);

clear:
  memset_ffs(r,0x00,RED_SIZE);
  return 1;
}

