#include "crypto.h"
#include "log.h"
#include "state.h"
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>

#define RNG_BORKTEST_SIZE              32

int rng_open(char const *path){
  uint8_t bork[RNG_BORKTEST_SIZE];
  int     fd;

  fd = open(path,O_RDONLY);
  if(fd == -1){
    log_errno("open");
    irclog_err("could not open rng '%s'",path);
    return -1;
  }

  if(!rng_read(fd,bork,sizeof(bork))){
    close(fd);
    return -1;
  }

  if(entropy_missing(bork,sizeof(bork))){
    irclog_err("rng '%s' is borken, all bytes were 0x%02X",
               path,bork[0]);
    close(fd);
    return -1;
  }

  return fd;
}

void rng_close(int fd){
  close(fd);
}

static ssize_t _rng_read(int fd,void *data,size_t size){
  char   *p       = data;
  size_t  p_size  = size;

  memset_ffs(p,0x00,p_size);

  do{
    ssize_t n = read(fd,p,p_size);
    if(n < 0)               return -1;
    if(n > (ssize_t)p_size) return  0;
    p       += n;
    p_size  -= n;
  }while(p_size > 0);

  return size;
}

bool rng_read(int fd,void *data,size_t size){
  ssize_t r = _rng_read(fd,data,size);
  if(r != (ssize_t)size){
    if(r == -1) log_errno("read");
    irclog_err("could not read %zu bytes from rng",size);
    return 0;
  }
  return 1;
}

bool rng_read_nolog(int fd,void *data,size_t size){
  return _rng_read(fd,data,size) == (ssize_t)size;
}

bool entropy_missing(uint8_t const *data,size_t size){
  uint8_t first = data[0];
  for(size_t x = 1;x < size;x++){
    if(data[x] != first) return 0;
  }
  return 1;
}

