#include "string.h"
#include <stdint.h>
#include <string.h>
#include <ctype.h>

extern char        *strzzcpy  (char       *restrict dst,
                               char const *restrict src,
                               size_t               dst_size,
                               size_t               src_size);
extern char        *strpcpy   (char       *restrict dst,
                               char const *restrict src,
                               char const *restrict dst_end);
extern char        *strpzcpy  (char       *restrict dst,
                               char const *restrict src,
                               char const *restrict dst_end,
                               size_t               src_size);

char *strzcpy(char        *restrict dst,
              char const  *restrict src,
              size_t                dst_size){

  assert(dst_size != 0);

  while(--dst_size){
    char c = *src++;
    if(!c) break;
    *dst++ = c;
  }
  *dst = 0;
  return dst;
}

char *strzcpy_quote(char        *restrict dst,
                    char const  *restrict src,
                    size_t                dst_size){

  assert(dst_size != 0);
    
  if(--dst_size == 0){
    *dst = 0;
    return dst;
  }
  *dst++ = '\'';
  
  while(--dst_size){
    char c = *src++;
    if(!c) break;
    if(c == '\''){
      if(--dst_size == 0) break;
      *dst++ = '\\';
    }
    *dst++ = c;
  }

  if(dst_size != 0){
    *dst++ = '\'';
  }
  
  *dst = 0;
  return dst;
}

int strcmpci(char const *restrict a,
             char const *restrict b){

  for(;*a && *b;a++,b++){
    char al = tolower(*a);
    char bl = tolower(*b);
    if(al < bl) return -1;   
    if(al > bl) return  1;   
  }
  if(*a < *b) return -1;   
  if(*a > *b) return  1;   
  return 0;
}

int strncmpci(char const *restrict a,
              char const *restrict b,
              size_t               n){

  size_t x;
  for(x = 0;*a && *b;a++,b++,x++){
    if(x >= n) return 0;
    char al = tolower(*a);
    char bl = tolower(*b);
    if(al < bl) return -1;   
    if(al > bl) return  1;   
  }
  if(x >= n)  return  0;
  if(*a < *b) return -1;
  if(*a > *b) return  1;
  return 0;
}

char *strchr0(char const *s,char c){
  while(*s && *s != c) s++;
  return (char *)(uintptr_t)s;
}

char *strnotchr0(char const *s,char n){
  for(char c;c = *s;s++){
    if(c != n) break;
  }
  return (char *)(uintptr_t)s;
}

static bool char_white(char c){
  return c == ' ' || c == '\t' || c == '\r' || c == '\n';
}

char *scan_white(char const *p){
  while(*p && !char_white(*p)) p++;
  return (char *)(uintptr_t)p;
}

char *scan_nowhite(char const *p){
  while(char_white(*p)) p++;
  return (char *)(uintptr_t)p;
}

char *scan_back_nowhite(char const *str,char const *p){
  while(char_white(*p)){
    if(p <= str) return 0;
    p--;
  }
  return (char *)(uintptr_t)p;
}

char *str_nstr(char const  *s,
               char const  *f,
               size_t       s_len){

  char    fc;
  char    sc;
  size_t  f_len;

  fc = *f++;
  if(fc){
    f_len = strlen(f);
    do{
      do{
        if(s_len < 1) return 0;
        sc = *s++;
        s_len--;
        if(!sc) return 0;
      }while(sc != fc);
      if(f_len > s_len) return 0;
    }while(strncmp(s,f,f_len) != 0);
    s--;
  }
  return (char *)(uintptr_t)s;
}

void str_tolower(char *str){
  for(char c;c = *str;str++){
    *str = tolower(c);
  }
}

void str_toupper(char *str){
  for(char c;c = *str;str++){
    *str = toupper(c);
  }
}

void str_stripnl(char *str){
  char *p = strchr(str,'\n');
  if(p) *p = 0;
}

char *str_trim(char *str){
  char *p;
  str = scan_nowhite(str);
  p   = str+strlen(str)-1;
  p   = scan_back_nowhite(str,p);
  if(p) p[1] = 0;
  return str;
}

size_t str_args(char *str,size_t argc_max,char **argv){
  char  *p;
  char  *l;
  size_t n;

  p = str;
  n = 0;
  p = scan_nowhite(p);
  l = p;
  while(*p){
    p = scan_white(p);
    if(n < argc_max) argv[n] = l;
    n++;
    if(!*p) break;
    *p++  = 0;
    p     = scan_nowhite(p);
    l     = p;
  }
  return n;
}

bool str_valid(char const  *str,
               size_t       min,
               size_t       max,
               char const  *charset){

  size_t len;

  len = strlen(str);
  if(min != (size_t)-1 && len < min) return 0;
  if(max != (size_t)-1 && len > max) return 0;

  if(charset && strspn(str,charset) != len) return 0;

  return 1;
}

