#define _POSIX_C_SOURCE 200112L
#include "time.h"
#include "config.h"
#include "def.h"
#include "state.h"
#include <time.h>
#include <stdio.h>
#include <assert.h>

static void gettime(clockid_t id,struct timespec *ts){
  int r = clock_gettime(id,ts);
  assert(r != -1); (void)r;
}

static void monotonic(struct timespec *ts){
  gettime(CLOCK_MONOTONIC,ts);
  if(mono_offset == (uint64_t)-1){
    mono_offset = time_sec()-(uint64_t)ts->tv_sec;
  }
}

uint64_t mono_sec(void){
  struct timespec ts;
  monotonic(&ts);
  return mono_offset+(uint64_t)ts.tv_sec;
}

uint64_t mono_nsec(void){
  struct timespec ts;
  uint64_t        t;
  monotonic(&ts);
  t  = mono_offset*NSEC_SEC;
  t += (uint64_t)ts.tv_sec*NSEC_SEC;
  t += (uint64_t)ts.tv_nsec;
  return t;
}

uint64_t time_sec(void){
  struct timespec ts;
  gettime(CLOCK_REALTIME,&ts);
  return ts.tv_sec;
}

size_t time_str(char       *buf,
                size_t      buf_size,
                char const *fmt,
                uint64_t    time){

  time_t      tt;
  struct tm  *info;
  size_t      n;

  tt    = time;
  info  = localtime(&tt);
  if(!info){
    buf[0] = 0;
    return 0;
  }

  n = strftime(buf,buf_size,fmt,info);
  if(!n){
    buf[0] = 0;
    return 0;
  }

  return n;
}

void sleep_nsec(size_t nsec){
  struct timespec t;
  t.tv_sec  = nsec/NSEC_SEC;
  t.tv_nsec = nsec%NSEC_SEC;
  nanosleep(&t,0);
}

void fmt_stampdiff(char      *buf,
                   size_t     buf_size,
                   uint64_t   a,
                   uint64_t   b){

  uint64_t    diff;
  char        sign;
  char const *unit;

  if(b >= a){
    diff = b-a;
    sign = '+';
  }else{
    diff = a-b;
    sign = '-';
  }

  if     ( diff < SEC_MIN  ) {                   unit = "second"; }
  else if( diff < SEC_HOUR ) { diff /= SEC_MIN;  unit = "minute"; }
  else if( diff < SEC_DAY  ) { diff /= SEC_HOUR; unit = "hour";   }
  else if( diff < SEC_YEAR ) { diff /= SEC_DAY;  unit = "day";    }
  else                       { diff /= SEC_YEAR; unit = "year";   }

  snprintf(buf,buf_size,"%c%lu %s%s",sign,diff,unit,diff == 1 ? "" : "s");
}

