#include "utf8.h"
#include <stdint.h>
#include <assert.h>

static bool utf8_cont(uint8_t c){
  return (c & 0xC0) == 0x80;
}

static bool unicode_valid(uint32_t u){
  return u <= 0x0010FFFF &&
         !(u >= 0x0000D800 &&
           u <= 0x0000DFFF);
}

static size_t utf8_readchar(uint8_t const *p,uint32_t *u_out){
  uint32_t  u;
  size_t    s;

  if((*p & 0x80) == 0x00){
    s = 1;
    u = p[0];
  }else if((*p & 0xE0) == 0xC0){
    if(!utf8_cont(p[1])) return 0;
    s = 2;
    u = (p[0] & 0x1F) <<  6 |
        (p[1] & 0x3F) <<  0;
    if(u < 0x00000080) return 0;
  }else if((*p & 0xF0) == 0xE0){
    if(!utf8_cont(p[1]) ||
       !utf8_cont(p[2])) return 0;
    s = 3;
    u = (p[0] & 0x0F) << 12 |
        (p[1] & 0x3F) <<  6 |
        (p[2] & 0x3F) <<  0;
    if(u < 0x00000800 ||
       (u >= 0x0000D800 &&
        u <= 0x0000DFFF)) return 0;
  }else if((*p & 0xF8) == 0xF0){
    if(!utf8_cont(p[1]) ||
       !utf8_cont(p[2]) ||
       !utf8_cont(p[3])) return 0;
    s = 4;
    u = (p[0] & 0x07) << 18 |
        (p[1] & 0x3F) << 12 |
        (p[2] & 0x3F) <<  6 |
        (p[3] & 0x3F) <<  0;
    if(u < 0x00010000 ||
       u > 0x0010FFFF) return 0;
  }else{
    return 0;
  }

  if(u_out) *u_out = u;
  return s;
}

static size_t utf8_charsize_unsafe(uint8_t const *p){
  if((*p & 0x80) == 0x00){
    return 1;
  }else if((*p & 0xE0) == 0xC0){
    return 2;
  }else if((*p & 0xF0) == 0xE0){
    return 3;
  }else if((*p & 0xF8) == 0xF0){
    return 4;
  }else{
    assert(0);
    return 1;
  }
}

bool utf8_valid(char const *ustr){
  uint8_t const *p = (uint8_t const *)ustr;

  while(*p){
    size_t s = utf8_readchar(p,0);
    if(s == 0) return 0;
    p += s;
  }

  return 1;
}

size_t utf8_cram_unsafe(char const *ustr,size_t buf_size){
  uint8_t const  *p = (uint8_t const *)ustr;
  size_t          b = buf_size;

  while(*p){
    size_t s = utf8_charsize_unsafe(p);
    if(s > b) break;
    p += s;
    b -= s;
  }

  return p-(uint8_t const *)ustr;
}

