#include "util.h"
#include "string.h"
#include "log.h"
#include "crypto.h"
#include "net.h"
#include "def.h"
#include "state.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

bool port_valid(int64_t port){
  return port >= PORT_MIN && port <= PORT_MAX;
}

bool check_port(int64_t port,char const *proto){
  if(!port_valid(port)){
    irclog_err("invalid %s port: %d (valid range: %d-%d)",
               proto,port,PORT_MIN,PORT_MAX);
    return 0;
  }
  return 1;
}

bool check_irc_user(char const *user){
  if(!str_valid(user,1,IRC_USER_LEN,HANDLE_CHARSET)){
    log_err("invalid IRC username: '%s'",user);
    return 0;
  }
  return 1;
}

bool handle_valid(char const *handle){
  return str_valid(handle,1,HANDLE_LEN,HANDLE_CHARSET);
}

void fmt_peerhost(char *buf,size_t buf_size,char const *handle){
  snprintf(buf,buf_size,"%s!%s@"IRC_PEERHOST,handle,handle);
}

void fmt_handle_ext(char         *buf,
                    size_t        buf_size,
                    char const   *speaker,
                    handle_t     *parens,
                    size_t        parens_cnt){

  char *b = buf;
  char *e = buf+buf_size;

  b = strpzcpy(b,speaker,e,SPEAKER_SIZE);
  b = strpcpy(b,"[",e);
  for(size_t x = 0;x < parens_cnt;x++){
    b = strpcpy(b,parens[x],e);
    b = strpcpy(b,x+1 < parens_cnt ? "|" : "]",e);
  }
}

void cpy_speaker(char *buf,size_t buf_size,uint8_t const *m){
  strzzcpy(buf,(char const *)(m+SPEAKER_OFF),buf_size,SPEAKER_SIZE);
}

void cpy_text(char *buf,size_t buf_size,uint8_t const *m){
  strzzcpy(buf,(char const *)(m+TEXT_OFF),buf_size,TEXT_SIZE);
}

bool index_shuffle(uint16_t *i,size_t n){
  uint16_t rngolade[n];

  assert(n <= 65536);
  if(!rng_read(rng_fast_fd,rngolade,sizeof(*rngolade)*n)) return 0;

  for(size_t x = 0;x < n;x++){
    i[x] = x;
  }

  for(size_t x = 0;x < n;x++){
    uint16_t y;
    uint16_t t;

    y     = rngolade[x]%n;
    t     = i[x];
    i[x]  = i[y];
    i[y]  = t;
  }

  return 1;
}

